@@ -34,14 +34,12 @@ class Database:
 
             
             if price_adj == False:
-                print(response.content)
-
                 ohlc_result = json.loads(json.loads(response.content).replace("'", "\"").replace('False','0').replace('True','1'))
             else:
 
-                #result = ast.literal_eval(json.loads(response.content))
                 result = json.loads(json.loads(response.content).replace("'", "\"").replace('[(', '\"[(').replace(')]',')]\"').replace('False','0').replace('True','1'))
                 ohlc_result = result[0]
+
                 qfq_result = result[1].replace('[','').replace(']','').replace(' ','')
                 qfq_result = ast.literal_eval(qfq_result)
                 qfq_df = pd.DataFrame(qfq_result, columns=['date', 'qfq_factor'])
@@ -81,21 +79,31 @@ class Database:
 
 
             if price_adj == True:
+
                 qfq_df = qfq_df[qfq_df['date'] > 20100000]
+
                 qfq_df = qfq_df.iloc[::-1].reset_index(drop=True)
                 qfq_df['date'] = qfq_df['date'].astype(str)
                 first_date = qfq_df.loc[qfq_df.index[0], 'date']
+                min_date_of_ohlc_df = df['date'].min()
+                if first_date > min_date_of_ohlc_df:
+                    first_date = min_date_of_ohlc_df
+
                 last_date = str(max([int(qfq_df.loc[qfq_df.index[-1], 'date']), int(df.loc[df.index[-1], 'date'])]))
                 qfq_df['date'] = pd.to_datetime(qfq_df['date'], format='%Y-%m-%d')
                 qfq_df = qfq_df.set_index(keys='date')
                 t_index = pd.DatetimeIndex(pd.date_range(start=first_date, end=last_date, freq='1D'))
                 qfq_df = qfq_df.reindex(t_index)
                 qfq_df = qfq_df.fillna(method='bfill')
+                qfq_df = qfq_df.fillna(method='ffill')
                 qfq_df = qfq_df.reset_index()
                 qfq_df['index'] = qfq_df['index'].astype(str)
                 qfq_df['index'] = qfq_df['index'].str.replace('-', '')
                 df = pd.merge(left=df, left_on='date', right=qfq_df, right_on='index')
 
+                if df['qfq_factor'].isnull().values.any():
+                    df['qfq_factor'] = df['qfq_factor'].fillna(method='ffill')
+
                 if 'T' in freq:
                     df['datetime'] = df['date'] + ' ' + df['time']
                     df['datetime'] = pd.to_datetime(df['datetime'], format='%Y%m%d %H%M%S')
@@ -155,7 +163,6 @@ class Database:
 
             response = requests.get(link_str)
 
-            #result = ast.literal_eval(json.loads(response.content))
             result = json.loads(json.loads(response.content).replace("'", "\""))
 
             df_list = []
@@ -218,7 +225,6 @@ class Database:
                 if year_month not in year_month_involved_list:
                     year_month_involved_list.append(year_month)
 
-            #if end_date >= int(expiry_date_dict[max(year_month_involved_list)].strftime('%Y%m%d')):
             year_month_involved_list.append((datetime.datetime.strptime(year_month_involved_list[-1], '%y%m') + relativedelta(months=1)).strftime('%y%m'))
 
             year_month_involved_list_str = json.dumps(str(year_month_involved_list))
@@ -361,7 +367,6 @@ class Database:
             df = df.sort_values(['date', 'market_cap_mil'], ascending=[True, False])
             df = df.reset_index(drop=True)
             df['date'] = pd.to_datetime(df['date'],format='%Y-%m-%d')
-            #df['record_date'] = pd.to_datetime(df['record_date'],format='%Y%m%d')
             df = df[['date','code','issued_share_mil','market_cap_mil','cumulative_market_cap_mil']]
             return df
 
@@ -401,7 +406,6 @@ class Database:
             df = df.sort_values(by='date')
             df = df.reset_index(drop=True)
             df['date'] = pd.to_datetime(df['date'],format='%Y-%m-%d')
-            #df['record_date'] = pd.to_datetime(df['record_date'],format='%Y%m%d')
             df = df[['date','code','issued_share_mil','market_cap_mil','cumulative_market_cap_mil']]
             df = df.set_index('date')
             return df
@@ -453,7 +457,6 @@ class Database:
             json_content = json_content.replace(' nan', '\" nan\"')
 
             result = ast.literal_eval(json_content)
-            #result = json.loads(json.loads(response.content).replace("'", "\""))
 
             df_list = []
             for item in result:
@@ -486,7 +489,6 @@ class Database:
         link_str = link_url + code_str + '&' + token_str + '&' + database_str
 
         response = requests.get(link_str)
-        #result = ast.literal_eval(json.loads(response.content))
         result = json.loads(json.loads(response.content).replace("'", "\""))
 
         df = pd.DataFrame(result)
@@ -616,8 +618,7 @@ class Database:
             df['date'] = pd.to_datetime(df['date'], format='%Y%m%d')
             df = df.set_index('date')
             df = df.sort_index()
-            #df = df.reset_index(drop=False)
-            
+
             for col in df.columns:
                 if 'participants_participant_' in col:
                     df = df.rename(columns={col:col.replace('participants_participant_','participants_')})
@@ -906,6 +907,7 @@ class Database:
             name_list.remove('hk_full_market')
 
         return name_list
+
     def get_all_hk_stock_plate_name(self):
 
         link_url = 'http://www.hkfdb.net/data_api?'
@@ -928,21 +930,30 @@ class Database:
         link_str = link_url + '&' + token_str + '&' + database_str
         
         response = requests.get(link_str)
-        json1 = json.loads(response.content).replace("'", "\"").replace('nan','NaN')
+        r_content = response.content
+        json1 = json.loads(r_content).replace("'", "\"")
+        json1 = json1.replace('nan','NaN')
+        json1 = json1.replace('INT\"L','INTL')
+        json1 = json1.replace('L\"OCCITANE','LOCCITANE')
+        json1 = json1.replace('True','\"True\"')
+        json1 = json1.replace('False','\"False\"')
+
         result = json.loads(json1)
 
         df = pd.DataFrame(result)
+
         df['ipo_date'] = pd.to_datetime(df['ipo_date'],format='%Y-%m-%d')
-        #df['ipo_date'] = df['ipo_date'].astype(str)
 
         df['share_issued'] = df['share_issued'].astype(str)
-        df['market_capital'] = df['market_capital'].astype(str)
-
         df['share_issued'] = df['share_issued'].str.replace('-','')
-        df['market_capital'] = df['market_capital'].str.replace('-','')
-        
-        df['share_issued'] = df['share_issued'].astype(int)
-        df['market_capital'] = df['market_capital'].astype(int)
+        df['share_issued'] = df['share_issued'].astype('int64')
+
+        df = df.sort_values(by='code')
+
+        df['market_capital'] = df['market_capital'].map(lambda x:round(x,0))
+        df['market_capital'] = df['market_capital'].astype('int64')
+
+        df = df[['code','stock_name','price','PE','EPS','dividend','volume','turnover','share_issued','lot_size','market_capital','ipo_date','year_end_date', 'is_stop_trading']]
 
         return df
 
@@ -1049,8 +1060,10 @@ class Database:
             df['date'] = pd.to_datetime(df['date'], format='%Y%m%d')
 
             df = df.set_index(keys='date')
-            #df = df.reset_index(drop=False)
             df = df.sort_index()
+            for col in df.columns:
+                df[col] = df[col].astype(int)
+
             return df
 
         else:
@@ -1106,7 +1119,6 @@ class Database:
             elif deri == 'fut':
                 df = df.sort_values(by='date')
                 df = df.set_index('date')
-                #df = df.reset_index(drop=True)
 
             if 'strike_price' in df.columns:
                 df = df.rename(columns={'strike_price': 'strike'})
